@@ -1,7 +1,9 @@
 #!/bin/sh
 
 # User Interface Events.
-# Copyright 1999, 2000, 2001 Free Software Foundation, Inc.
+#
+# Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2007
+# Free Software Foundation, Inc.
 #
 # Contributed by Cygnus Solutions.
 #
@@ -9,7 +11,7 @@
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -18,26 +20,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-
-#
-# What happens next:
-#
-
-# The gdb-events.h/gdb-events.c files this script generates are commited
-# and published.
-
-# Any UI module that is installing events is changed so that the
-# events are installed using the ``set_gdb_events()'' and
-# ``gdb_event_hooks()'' interfaces.  There could prove to be an issue
-# here with respect to annotate.  We might need to accomodate a hook
-# stack that allows several ui blocks to install their own events.
-
-# Each of the variable events (as currently generated) is converted
-# to either a straight function call or a function call with a
-# predicate.
-
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 IFS=:
 
@@ -65,46 +48,6 @@ f:void:tracepoint_create:int number:number
 f:void:tracepoint_delete:int number:number
 f:void:tracepoint_modify:int number:number
 f:void:architecture_changed:void
-#*:void:annotate_starting_hook:void
-#*:void:annotate_stopped_hook:void
-#*:void:annotate_signalled_hook:void
-#*:void:annotate_signal_hook:void
-#*:void:annotate_exited_hook:void
-##*:void:print_register_hook:int
-##*:CORE_ADDR:find_toc_address_hook:CORE_ADDR
-##*:void:sparc_print_register_hook:int regno:regno
-#*:void:target_resume_hook:void
-#*:void:target_wait_loop_hook:void
-#*:void:init_gdb_hook:char *argv0:argv0
-#*:void:command_loop_hook:void
-#*:void:fputs_unfiltered_hook:const char *linebuff,struct ui_file *stream:linebuff, stream
-#*:void:print_frame_info_listing_hook:struct symtab *s, int line, int stopline, int noerror:s, line, stopline, noerror
-#*:int:query_hook:const char *query, va_list args:query, args
-#*:void:warning_hook:const char *string, va_list args:string, args
-#*:void:target_output_hook:char *b:b
-#*:void:interactive_hook:void
-#*:void:registers_changed_hook:void
-#*:void:readline_begin_hook:char *format, ...:format
-#*:char *:readline_hook:char *prompt:prompt
-#*:void:readline_end_hook:void
-#*:void:register_changed_hook:int regno:regno
-#*:void:memory_changed_hook:CORE_ADDR addr, int len:addr, len
-#*:void:context_hook:int num:num
-#*:int:target_wait_hook:int pid, struct target_waitstatus *status:pid, status
-#*:void:call_command_hook:struct cmd_list_element *c, char *cmd, int from_tty:c, cmd, from_tty
-#*:NORETURN void:error_hook:void:: ATTR_NORETURN
-#*:void:error_begin_hook:void
-##*:int:target_architecture_hook:const struct bfd_arch_info *
-#*:void:exec_file_display_hook:char *filename:filename
-#*:void:file_changed_hook:char *filename:filename
-##*:void:specify_exec_file_hook:
-#*:int:gdb_load_progress_hook:char *section, unsigned long num:section, num
-#*:void:pre_add_symbol_hook:char *name:name
-#*:void:post_add_symbol_hook:void
-#*:void:selected_frame_level_changed_hook:int level:level
-#*:int:gdb_loop_hook:int signo:signo
-##*:void:solib_create_inferior_hook:void
-##*:void:xcoff_relocate_symtab_hook:unsigned int
 EOF
   grep -v '^#'
 }
@@ -113,7 +56,9 @@ copyright ()
 {
   cat <<EOF
 /* User Interface Events.
-   Copyright 1999, 2001 Free Software Foundation, Inc.
+
+   Copyright (C) 1999, 2001, 2002, 2004, 2005, 2007
+   Free Software Foundation, Inc.
 
    Contributed by Cygnus Solutions.
 
@@ -121,17 +66,16 @@ copyright ()
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Work in progress */
 
@@ -145,7 +89,7 @@ copyright ()
    If editing this file, please also run gdb-events.sh and merge any
    changes into that script. Conversely, when making sweeping changes
    to this file, modifying gdb-events.sh and using its output may
-   prove easier. */
+   prove easier.  */
 
 EOF
 }
@@ -160,10 +104,6 @@ cat <<EOF
 
 #ifndef GDB_EVENTS_H
 #define GDB_EVENTS_H
-
-#ifndef WITH_GDB_EVENTS
-#define WITH_GDB_EVENTS 1
-#endif
 EOF
 
 # pointer declarations
@@ -187,9 +127,9 @@ done
 echo ""
 echo ""
 cat <<EOF
-/* Type definition of all hook functions.
-   Recommended pratice is to first declare each hook function using
-   the below ftype and then define it. */
+/* Type definition of all hook functions.  Recommended pratice is to
+   first declare each hook function using the below ftype and then
+   define it.  */
 EOF
 echo ""
 function_list | while eval read $read
@@ -218,7 +158,7 @@ echo ""
 cat <<EOF
 /* Interface into events functions.
    Where a *_p() predicate is present, it must be called before
-   calling the hook proper. */
+   calling the hook proper.  */
 EOF
 function_list | while eval read $read
 do
@@ -234,43 +174,17 @@ do
   esac
 done
 
-# function macros
-echo ""
-echo ""
-cat <<EOF
-/* When GDB_EVENTS are not being used, completly disable them. */
-EOF
-echo ""
-echo "#if !WITH_GDB_EVENTS"
-function_list | while eval read $read
-do
-  case "${class}" in
-    "*" ) continue ;;
-    "?" )
-	echo "#define ${function}_event_p() 0"
-	echo "#define ${function}_event(${actual}) 0"
-	;;
-    "f" )
-	echo "#define ${function}_event(${actual}) 0"
-	;;
-  esac
-done
-echo "#endif"
-
 # our set function
 cat <<EOF
 
-/* Install custom gdb-events hooks. */
-extern struct gdb_events *set_gdb_event_hooks (struct gdb_events *vector);
+/* Install custom gdb-events hooks.  */
+extern struct gdb_events *deprecated_set_gdb_event_hooks (struct gdb_events *vector);
 
-/* Deliver any pending events. */
+/* Deliver any pending events.  */
 extern void gdb_events_deliver (struct gdb_events *vector);
 
-#if !WITH_GDB_EVENTS
-#define set_gdb_events(x) 0
-#define set_gdb_event_hooks(x) 0
-#define gdb_events_deliver(x) 0
-#endif
+/* Clear event handlers.  */
+extern void clear_gdb_event_hooks (void);
 EOF
 
 # close it off
@@ -303,32 +217,21 @@ cat <<EOF
 #include "gdb-events.h"
 #include "gdbcmd.h"
 
-#undef XMALLOC
-#define XMALLOC(TYPE) ((TYPE*) xmalloc (sizeof (TYPE)))
-
-#if WITH_GDB_EVENTS
 static struct gdb_events null_event_hooks;
 static struct gdb_events queue_event_hooks;
 static struct gdb_events *current_event_hooks = &null_event_hooks;
-#endif
 
 int gdb_events_debug;
+static void
+show_gdb_events_debug (struct ui_file *file, int from_tty,
+                       struct cmd_list_element *c, const char *value)
+{
+  fprintf_filtered (file, _("Event debugging is %s.\\n"), value);
+}
+
 EOF
 
-# global pointer variables - always have this
-#echo ""
-#function_list | while eval read $read
-#do
-#  case "${class}" in
-#    "*" )
-#	echo "${returntype} (*${function}_event) (${formal})${attrib} = 0;"
-#	;;
-#  esac
-#done
-
 # function bodies
-echo ""
-echo "#if WITH_GDB_EVENTS"
 function_list | while eval read $read
 do
   case "${class}" in
@@ -365,15 +268,12 @@ EOF
 	;;
   esac
 done
-echo ""
-echo "#endif"
 
 # Set hooks function
 echo ""
 cat <<EOF
-#if WITH_GDB_EVENTS
 struct gdb_events *
-set_gdb_event_hooks (struct gdb_events *vector)
+deprecated_set_gdb_event_hooks (struct gdb_events *vector)
 {
   struct gdb_events *old_events = current_event_hooks;
   if (vector == NULL)
@@ -392,7 +292,16 @@ do
 done
 cat <<EOF
 }
-#endif
+EOF
+
+# Clear hooks function
+echo ""
+cat <<EOF
+void
+clear_gdb_event_hooks (void)
+{
+  deprecated_set_gdb_event_hooks (&null_event_hooks);
+}
 EOF
 
 # event type
@@ -558,7 +467,6 @@ void
 _initialize_gdb_events (void)
 {
   struct cmd_list_element *c;
-#if WITH_GDB_EVENTS
 EOF
 function_list | while eval read $read
 do
@@ -569,21 +477,15 @@ do
   esac
 done
 cat <<EOF
-#endif
 
-  c = add_set_cmd ("eventdebug", class_maintenance, var_zinteger,
-		   (char *) (&gdb_events_debug), "Set event debugging.\n\\
-When non-zero, event/notify debugging is enabled.", &setlist);
-  deprecate_cmd (c, "set debug event");
-  deprecate_cmd (add_show_from_set (c, &showlist), "show debug event");
-
-  add_show_from_set (add_set_cmd ("event",
-                                  class_maintenance,
-                                  var_zinteger,
-                                  (char *) (&gdb_events_debug),
-                                  "Set event debugging.\n\\
-When non-zero, event/notify debugging is enabled.", &setdebuglist),
-		     &showdebuglist);
+  add_setshow_zinteger_cmd ("event", class_maintenance,
+                            &gdb_events_debug, _("\\
+Set event debugging."), _("\\
+Show event debugging."), _("\\
+When non-zero, event/notify debugging is enabled."),
+                            NULL,
+                            show_gdb_events_debug,
+                            &setdebuglist, &showdebuglist);
 }
 EOF
 
