@@ -1,12 +1,12 @@
 /* C preprocessor macro tables for GDB.
-   Copyright 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2007 Free Software Foundation, Inc.
    Contributed by Red Hat, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "gdb_obstack.h"
@@ -426,11 +424,10 @@ macro_include (struct macro_source_file *source,
   struct macro_source_file **link;
 
   /* Find the right position in SOURCE's `includes' list for the new
-     file.  Scan until we find the first file we shouldn't follow ---
-     which is therefore the file we should directly precede --- or
-     reach the end of the list.  */
+     file.  Skip inclusions at earlier lines, until we find one at the
+     same line or later --- or until the end of the list.  */
   for (link = &source->includes;
-       *link && line < (*link)->included_at_line;
+       *link && (*link)->included_at_line < line;
        link = &(*link)->next_included)
     ;
 
@@ -446,7 +443,7 @@ macro_include (struct macro_source_file *source,
 
          First, squawk.  */
       complaint (&symfile_complaints,
-		 "both `%s' and `%s' allegedly #included at %s:%d", included,
+		 _("both `%s' and `%s' allegedly #included at %s:%d"), included,
 		 (*link)->filename, source->filename, line);
 
       /* Now, choose a new, unoccupied line number for this
@@ -705,7 +702,7 @@ check_for_redefinition (struct macro_source_file *source, int line,
       if (! same)
         {
 	  complaint (&symfile_complaints,
-		     "macro `%s' redefined at %s:%d; original definition at %s:%d",
+		     _("macro `%s' redefined at %s:%d; original definition at %s:%d"),
 		     name, source->filename, line,
 		     found_key->start_file->filename, found_key->start_line);
         }
@@ -794,7 +791,7 @@ macro_undef (struct macro_source_file *source, int line,
       if (key->end_file)
         {
 	  complaint (&symfile_complaints,
-		     "macro '%s' is #undefined twice, at %s:%d and %s:%d", name,
+		     _("macro '%s' is #undefined twice, at %s:%d and %s:%d"), name,
 		     source->filename, line, key->end_file->filename,
 		     key->end_line);
         }
@@ -811,7 +808,7 @@ macro_undef (struct macro_source_file *source, int line,
          ignore it too.  */
 #if 0
       complaint (&symfile_complaints,
-		 "no definition for macro `%s' in scope to #undef at %s:%d",
+		 _("no definition for macro `%s' in scope to #undef at %s:%d"),
 		 name, source->filename, line);
 #endif
     }
