@@ -1,6 +1,6 @@
 /* MI Command Set - varobj commands.
 
-   Copyright (C) 2000, 2002, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2002, 2004, 2005, 2007 Free Software Foundation, Inc.
 
    Contributed by Cygnus Solutions (a Red Hat company).
 
@@ -8,7 +8,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -17,9 +17,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "mi-cmds.h"
@@ -34,10 +32,43 @@ const char mi_no_values[] = "--no-values";
 const char mi_simple_values[] = "--simple-values";
 const char mi_all_values[] = "--all-values";
 
-extern int varobjdebug;		/* defined in varobj.c */
+extern int varobjdebug;		/* defined in varobj.c.  */
 
-static int varobj_update_one (struct varobj *var,
-			      enum print_values print_values);
+static void varobj_update_one (struct varobj *var,
+			      enum print_values print_values,
+			      int explicit);
+
+static int mi_print_value_p (struct type *type, enum print_values print_values);
+
+/* Print variable object VAR.  The PRINT_VALUES parameter controls
+   if the value should be printed.  The PRINT_EXPRESSION parameter
+   controls if the expression should be printed.  */
+static void 
+print_varobj (struct varobj *var, enum print_values print_values,
+	      int print_expression)
+{
+  struct type *gdb_type;
+  char *type;
+
+  ui_out_field_string (uiout, "name", varobj_get_objname (var));
+  if (print_expression)
+    ui_out_field_string (uiout, "exp", varobj_get_expression (var));
+  ui_out_field_int (uiout, "numchild", varobj_get_num_children (var));
+  
+  gdb_type = varobj_get_gdb_type (var);
+  if (gdb_type && mi_print_value_p (gdb_type, print_values))
+    ui_out_field_string (uiout, "value", varobj_get_value (var));
+
+  type = varobj_get_type (var);
+  if (type != NULL)
+    {
+      ui_out_field_string (uiout, "type", type);
+      xfree (type);
+    }
+
+  if (varobj_get_frozen (var))
+    ui_out_field_int (uiout, "frozen", 1);
+}
 
 /* VAROBJ operations */
 
@@ -49,7 +80,6 @@ mi_cmd_var_create (char *command, char **argv, int argc)
   char *name;
   char *frame;
   char *expr;
-  char *type;
   struct cleanup *old_cleanups;
   enum varobj_type var_type;
 
@@ -66,9 +96,10 @@ mi_cmd_var_create (char *command, char **argv, int argc)
   old_cleanups = make_cleanup (free_current_contents, &name);
 
   frame = xstrdup (argv[1]);
-  old_cleanups = make_cleanup (xfree, frame);
+  make_cleanup (xfree, frame);
 
   expr = xstrdup (argv[2]);
+  make_cleanup (xfree, expr);
 
   if (strcmp (name, "-") == 0)
     {
@@ -98,16 +129,7 @@ mi_cmd_var_create (char *command, char **argv, int argc)
   if (var == NULL)
     error (_("mi_cmd_var_create: unable to create variable object"));
 
-  ui_out_field_string (uiout, "name", name);
-  ui_out_field_int (uiout, "numchild", varobj_get_num_children (var));
-  type = varobj_get_type (var);
-  if (type == NULL)
-    ui_out_field_string (uiout, "type", "");
-  else
-    {
-      ui_out_field_string (uiout, "type", type);
-      xfree (type);
-    }
+  print_varobj (var, PRINT_ALL_VALUES, 0 /* don't print expression */);
 
   do_cleanups (old_cleanups);
   return MI_CMD_DONE;
@@ -117,7 +139,6 @@ enum mi_cmd_result
 mi_cmd_var_delete (char *command, char **argv, int argc)
 {
   char *name;
-  char *expr;
   struct varobj *var;
   int numdel;
   int children_only_p = 0;
@@ -145,13 +166,12 @@ mi_cmd_var_delete (char *command, char **argv, int argc)
      which would be the variable name. */
   if (argc == 2)
     {
-      expr = xstrdup (argv[1]);
       if (strcmp (name, "-c") != 0)
 	error (_("mi_cmd_var_delete: Invalid option."));
       children_only_p = 1;
-      xfree (name);
-      name = xstrdup (expr);
-      xfree (expr);
+      do_cleanups (old_cleanups);
+      name = xstrdup (argv[1]);
+      make_cleanup (free_current_contents, &name);
     }
 
   /* If we didn't error out, now NAME contains the name of the
@@ -187,7 +207,7 @@ mi_cmd_var_set_format (char *command, char **argv, int argc)
   if (var == NULL)
     error (_("mi_cmd_var_set_format: Variable object not found"));
 
-  formspec = xstrdup (argv[1]);
+  formspec = argv[1];
   if (formspec == NULL)
     error (_("mi_cmd_var_set_format: Must specify the format as: \"natural\", \"binary\", \"decimal\", \"hexadecimal\", or \"octal\""));
 
@@ -214,6 +234,35 @@ mi_cmd_var_set_format (char *command, char **argv, int argc)
   return MI_CMD_DONE;
 }
 
+enum mi_cmd_result
+mi_cmd_var_set_frozen (char *command, char **argv, int argc)
+{
+  struct varobj *var;
+  int frozen;
+
+  if (argc != 2)
+    error (_("-var-set-format: Usage: NAME FROZEN_FLAG."));
+
+  var = varobj_get_handle (argv[0]);
+  if (var == NULL)
+    error (_("Variable object not found"));
+
+  if (strcmp (argv[1], "0") == 0)
+    frozen = 0;
+  else if (strcmp (argv[1], "1") == 0)
+    frozen = 1;
+  else
+    error (_("Invalid flag value"));
+
+  varobj_set_frozen (var, frozen);
+
+  /* We don't automatically return the new value, or what varobjs got new
+     values during unfreezing.  If this information is required, client
+     should call -var-update explicitly.  */
+  return MI_CMD_DONE;
+}
+
+
 enum mi_cmd_result
 mi_cmd_var_show_format (char *command, char **argv, int argc)
 {
@@ -278,8 +327,7 @@ Must be: 0 or \"%s\", 1 or \"%s\", 2 or \"%s\""),
 static int
 mi_print_value_p (struct type *type, enum print_values print_values)
 {
-  if (type != NULL)
-    type = check_typedef (type);
+  type = check_typedef (type);
 
   if (print_values == PRINT_NO_VALUES)
     return 0;
@@ -303,7 +351,6 @@ mi_cmd_var_list_children (char *command, char **argv, int argc)
   struct varobj **cc;
   struct cleanup *cleanup_children;
   int numchild;
-  char *type;
   enum print_values print_values;
 
   if (argc != 1 && argc != 2)
@@ -325,7 +372,10 @@ mi_cmd_var_list_children (char *command, char **argv, int argc)
     print_values = PRINT_NO_VALUES;
 
   if (numchild <= 0)
-    return MI_CMD_DONE;
+    {
+      xfree (childlist);
+      return MI_CMD_DONE;
+    }
 
   if (mi_version (uiout) == 1)
     cleanup_children = make_cleanup_ui_out_tuple_begin_end (uiout, "children");
@@ -336,17 +386,9 @@ mi_cmd_var_list_children (char *command, char **argv, int argc)
     {
       struct cleanup *cleanup_child;
       cleanup_child = make_cleanup_ui_out_tuple_begin_end (uiout, "child");
-      ui_out_field_string (uiout, "name", varobj_get_objname (*cc));
-      ui_out_field_string (uiout, "exp", varobj_get_expression (*cc));
-      ui_out_field_int (uiout, "numchild", varobj_get_num_children (*cc));
-      if (mi_print_value_p (varobj_get_gdb_type (*cc), print_values))
-	ui_out_field_string (uiout, "value", varobj_get_value (*cc));
-      type = varobj_get_type (*cc);
-      /* C++ pseudo-variables (public, private, protected) do not have a type */
-      if (type)
-	ui_out_field_string (uiout, "type", type);
-      do_cleanups (cleanup_child);
+      print_varobj (*cc, print_values, 1 /* print expression */);
       cc++;
+      do_cleanups (cleanup_child);
     }
   do_cleanups (cleanup_children);
   xfree (childlist);
@@ -370,6 +412,27 @@ mi_cmd_var_info_type (char *command, char **argv, int argc)
   return MI_CMD_DONE;
 }
 
+enum mi_cmd_result
+mi_cmd_var_info_path_expression (char *command, char **argv, int argc)
+{
+  struct varobj *var;
+  char *path_expr;
+
+  if (argc != 1)
+    error (_("Usage: NAME."));
+
+  /* Get varobj handle, if a valid var obj name was specified.  */
+  var = varobj_get_handle (argv[0]);
+  if (var == NULL)
+    error (_("Variable object not found"));
+  
+  path_expr = varobj_get_path_expr (var);
+
+  ui_out_field_string (uiout, "path_expr", path_expr);
+
+  return MI_CMD_DONE;
+}
+
 enum mi_cmd_result
 mi_cmd_var_info_expression (char *command, char **argv, int argc)
 {
@@ -455,7 +518,7 @@ mi_cmd_var_assign (char *command, char **argv, int argc)
   expression = xstrdup (argv[1]);
 
   if (!varobj_set_value (var, expression))
-    error (_("mi_cmd_var_assign: Could not assign expression to varible object"));
+    error (_("mi_cmd_var_assign: Could not assign expression to variable object"));
 
   ui_out_field_string (uiout, "value", varobj_get_value (var));
   return MI_CMD_DONE;
@@ -491,10 +554,11 @@ mi_cmd_var_update (char *command, char **argv, int argc)
   if ((*name == '*') && (*(name + 1) == '\0'))
     {
       nv = varobj_list (&rootlist);
+      cleanup = make_cleanup (xfree, rootlist);
       if (mi_version (uiout) <= 1)
-        cleanup = make_cleanup_ui_out_tuple_begin_end (uiout, "changelist");
+        make_cleanup_ui_out_tuple_begin_end (uiout, "changelist");
       else
-        cleanup = make_cleanup_ui_out_list_begin_end (uiout, "changelist");
+        make_cleanup_ui_out_list_begin_end (uiout, "changelist");
       if (nv <= 0)
 	{
 	  do_cleanups (cleanup);
@@ -503,10 +567,9 @@ mi_cmd_var_update (char *command, char **argv, int argc)
       cr = rootlist;
       while (*cr != NULL)
 	{
-	  varobj_update_one (*cr, print_values);
+	  varobj_update_one (*cr, print_values, 0 /* implicit */);
 	  cr++;
 	}
-      xfree (rootlist);
       do_cleanups (cleanup);
     }
   else
@@ -520,57 +583,57 @@ mi_cmd_var_update (char *command, char **argv, int argc)
         cleanup = make_cleanup_ui_out_tuple_begin_end (uiout, "changelist");
       else
         cleanup = make_cleanup_ui_out_list_begin_end (uiout, "changelist");
-      varobj_update_one (var, print_values);
+      varobj_update_one (var, print_values, 1 /* explicit */);
       do_cleanups (cleanup);
     }
     return MI_CMD_DONE;
 }
 
-/* Helper for mi_cmd_var_update() Returns 0 if the update for
-   the variable fails (usually because the variable is out of
-   scope), and 1 if it succeeds. */
+/* Helper for mi_cmd_var_update().  */
 
-static int
-varobj_update_one (struct varobj *var, enum print_values print_values)
+static void
+varobj_update_one (struct varobj *var, enum print_values print_values,
+		   int explicit)
 {
   struct varobj **changelist;
   struct varobj **cc;
   struct cleanup *cleanup = NULL;
   int nc;
 
-  nc = varobj_update (&var, &changelist);
+  nc = varobj_update (&var, &changelist, explicit);
 
-  /* nc == 0 means that nothing has changed.
-     nc == -1 means that an error occured in updating the variable.
-     nc == -2 means the variable has changed type. */
+  /* nc >= 0  represents the number of changes reported into changelist.
+     nc < 0   means that an error occured or the the variable has 
+              changed type (TYPE_CHANGED).  */
   
   if (nc == 0)
-    return 1;
-  else if (nc == -1)
+    return;
+  else if (nc < 0)
     {
       if (mi_version (uiout) > 1)
         cleanup = make_cleanup_ui_out_tuple_begin_end (uiout, NULL);
       ui_out_field_string (uiout, "name", varobj_get_objname(var));
-      ui_out_field_string (uiout, "in_scope", "false");
-      if (mi_version (uiout) > 1)
-        do_cleanups (cleanup);
-      return -1;
-    }
-  else if (nc == -2)
-    {
-      if (mi_version (uiout) > 1)
-        cleanup = make_cleanup_ui_out_tuple_begin_end (uiout, NULL);
-      ui_out_field_string (uiout, "name", varobj_get_objname (var));
-      ui_out_field_string (uiout, "in_scope", "true");
-      ui_out_field_string (uiout, "new_type", varobj_get_type(var));
-      ui_out_field_int (uiout, "new_num_children", 
-			   varobj_get_num_children(var));
+
+      switch (nc)
+      {
+        case NOT_IN_SCOPE:
+          ui_out_field_string (uiout, "in_scope", "false");
+	  break;
+        case INVALID:
+          ui_out_field_string (uiout, "in_scope", "invalid");
+ 	  break;
+        case TYPE_CHANGED:
+	  ui_out_field_string (uiout, "in_scope", "true");
+          ui_out_field_string (uiout, "new_type", varobj_get_type(var));
+          ui_out_field_int (uiout, "new_num_children", 
+			    varobj_get_num_children(var));
+	  break;
+      }
       if (mi_version (uiout) > 1)
         do_cleanups (cleanup);
     }
   else
     {
-      
       cc = changelist;
       while (*cc != NULL)
 	{
@@ -586,7 +649,5 @@ varobj_update_one (struct varobj *var, enum print_values print_values)
 	  cc++;
 	}
       xfree (changelist);
-      return 1;
     }
-  return 1;
 }
