@@ -1,12 +1,12 @@
 /* MI Command Set - MI Command Parser.
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2007 Free Software Foundation, Inc.
    Contributed by Cygnus Solutions (a Red Hat company).
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,15 +15,22 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifndef MI_PARSE_H
 #define MI_PARSE_H
 
+#include <sys/time.h>
+
 /* MI parser */
 
+/* Timestamps for current command and last asynchronous command.  */
+struct mi_timestamp {
+  struct timeval wallclock;
+  struct timeval utime;
+  struct timeval stime;
+};
+
 enum mi_command_type
   {
     MI_COMMAND, CLI_COMMAND
@@ -35,6 +42,7 @@ struct mi_parse
     char *command;
     char *token;
     const struct mi_cmd *cmd;
+    struct mi_timestamp *cmd_start;
     char *args;
     char **argv;
     int argc;
