@@ -1,13 +1,13 @@
 /* Shared library declarations for GDB, the GNU Debugger.
    
-   Copyright (C) 1992, 1993, 1995, 1998, 1999, 2000, 2001, 2003, 2005 
+   Copyright (C) 1992, 1993, 1995, 1998, 1999, 2000, 2001, 2003, 2005, 2007
    Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,9 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifndef SOLIB_H
 #define SOLIB_H
@@ -26,6 +24,7 @@
 /* Forward decl's for prototypes */
 struct so_list;
 struct target_ops;
+struct target_so_ops;
 
 /* Called when we free all symtabs, to free the shared library information
    as well. */
@@ -57,4 +56,9 @@ extern int in_solib_dynsym_resolve_code (CORE_ADDR);
 
 extern void no_shared_libraries (char *ignored, int from_tty);
 
+/* Set the solib operations for GDBARCH to NEW_OPS.  */
+
+extern void set_solib_ops (struct gdbarch *gdbarch,
+			   struct target_so_ops *new_ops);
+
 #endif /* SOLIB_H */
