@@ -257,6 +257,15 @@ func_type v_func_type;
    will all xfail.  */
 
 
+/***********/
+
+typedef int foo;
+
+foo intfoo (afoo)
+{
+  return (afoo * 2);
+}
+
 /***********/
 
 int main ()
