@@ -1,7 +1,7 @@
 /* General functions for the WDB TUI.
 
-   Copyright 1998, 1999, 2000, 2001, 2002, 2003 Free Software Foundation,
-   Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2007
+   Free Software Foundation, Inc.
 
    Contributed by Hewlett-Packard Company.
 
@@ -9,7 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,35 +18,20 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <malloc.h>
-#ifdef HAVE_TERM_H
-#include <term.h>
-#endif
-#include <signal.h>
-#include <fcntl.h>
-#if 0
-#include <termio.h>
-#endif
-#include <setjmp.h>
 #include "defs.h"
 #include "gdbcmd.h"
-#include "tui.h"
-#include "tuiData.h"
-#include "tuiLayout.h"
-#include "tuiIO.h"
-#include "tuiRegs.h"
-#include "tuiStack.h"
-#include "tuiWin.h"
-#include "tuiSourceWin.h"
-#include "tuiDataWin.h"
-#include "readline/readline.h"
+#include "tui/tui.h"
+#include "tui/tui-hooks.h"
+#include "tui/tui-data.h"
+#include "tui/tui-layout.h"
+#include "tui/tui-io.h"
+#include "tui/tui-regs.h"
+#include "tui/tui-stack.h"
+#include "tui/tui-win.h"
+#include "tui/tui-winsource.h"
+#include "tui/tui-windata.h"
 #include "target.h"
 #include "frame.h"
 #include "breakpoint.h"
@@ -54,27 +39,37 @@
 #include "symtab.h"
 #include "source.h"
 
-#ifdef HAVE_NCURSES_H       
-#include <ncurses.h>
-#else
-#ifdef HAVE_CURSES_H
-#include <curses.h>
-#endif
+#include <stdio.h>
+#include <stdlib.h>
+#include <ctype.h>
+#include <signal.h>
+#include <fcntl.h>
+#if 0
+#include <termio.h>
 #endif
+#include <setjmp.h>
+
+#include "gdb_curses.h"
+
+/* This redefines CTRL if it is not already defined, so it must come
+   after terminal state releated include files like <term.h> and
+   "gdb_curses.h".  */
+#include "readline/readline.h"
 
 /* Tells whether the TUI is active or not.  */
 int tui_active = 0;
 static int tui_finish_init = 1;
 
-enum tui_key_mode tui_current_key_mode = tui_command_mode;
+enum tui_key_mode tui_current_key_mode = TUI_COMMAND_MODE;
 
 struct tui_char_command
 {
   unsigned char key;
-  const char* cmd;
+  const char *cmd;
 };
 
-/* Key mapping to gdb commands when the TUI is using the single key mode.  */
+/* Key mapping to gdb commands when the TUI is using the single key
+   mode.  */
 static const struct tui_char_command tui_commands[] = {
   { 'c', "continue" },
   { 'd', "down" },
@@ -107,19 +102,22 @@ tui_rl_switch_mode (int notused1, int notused2)
       tui_enable ();
     }
 
-  /* Clear the readline in case switching occurred in middle of something.  */
+  /* Clear the readline in case switching occurred in middle of
+     something.  */
   if (rl_end)
     rl_kill_text (0, rl_end);
 
   /* Since we left the curses mode, the terminal mode is restored to
      some previous state.  That state may not be suitable for readline
      to work correctly (it may be restored in line mode).  We force an
-     exit of the current readline so that readline is re-entered and it
-     will be able to setup the terminal for its needs.  By re-entering
-     in readline, we also redisplay its prompt in the non-curses mode.  */
+     exit of the current readline so that readline is re-entered and
+     it will be able to setup the terminal for its needs.  By
+     re-entering in readline, we also redisplay its prompt in the
+     non-curses mode.  */
   rl_newline (1, '\n');
 
-  /* Make sure the \n we are returning does not repeat the last command.  */
+  /* Make sure the \n we are returning does not repeat the last
+     command.  */
   dont_repeat ();
   return 0;
 }
@@ -127,23 +125,23 @@ tui_rl_switch_mode (int notused1, int notused2)
 /* TUI readline command.
    Change the TUI layout to show a next layout.
    This function is bound to CTRL-X 2.  It is intended to provide
-   a functionality close to the Emacs split-window command.  We always
-   show two windows (src+asm), (src+regs) or (asm+regs).  */
+   a functionality close to the Emacs split-window command.  We
+   always show two windows (src+asm), (src+regs) or (asm+regs).  */
 static int
 tui_rl_change_windows (int notused1, int notused2)
 {
   if (!tui_active)
-    tui_rl_switch_mode (0/*notused*/, 0/*notused*/);
+    tui_rl_switch_mode (0 /* notused */, 0 /* notused */);
 
   if (tui_active)
     {
-      TuiLayoutType new_layout;
-      TuiRegisterDisplayType regs_type = TUI_UNDEFINED_REGS;
+      enum tui_layout_type new_layout;
+      enum tui_register_display_type regs_type = TUI_UNDEFINED_REGS;
 
-      new_layout = currentLayout ();
+      new_layout = tui_current_layout ();
 
-      /* Select a new layout to have a rolling layout behavior
-	 with always two windows (except when undefined).  */
+      /* Select a new layout to have a rolling layout behavior with
+	 always two windows (except when undefined).  */
       switch (new_layout)
 	{
 	case SRC_COMMAND:
@@ -170,7 +168,7 @@ tui_rl_change_windows (int notused1, int notused2)
 	  new_layout = SRC_COMMAND;
 	  break;
 	}
-      tuiSetLayout (new_layout, regs_type);
+      tui_set_layout (new_layout, regs_type);
     }
   return 0;
 }
@@ -181,14 +179,14 @@ static int
 tui_rl_delete_other_windows (int notused1, int notused2)
 {
   if (!tui_active)
-    tui_rl_switch_mode (0/*notused*/, 0/*notused*/);
+    tui_rl_switch_mode (0 /* notused */, 0 /* notused */);
 
   if (tui_active)
     {
-      TuiLayoutType new_layout;
-      TuiRegisterDisplayType regs_type = TUI_UNDEFINED_REGS;
+      enum tui_layout_type new_layout;
+      enum tui_register_display_type regs_type = TUI_UNDEFINED_REGS;
 
-      new_layout = currentLayout ();
+      new_layout = tui_current_layout ();
 
       /* Kill one window.  */
       switch (new_layout)
@@ -205,7 +203,7 @@ tui_rl_delete_other_windows (int notused1, int notused2)
 	  new_layout = DISASSEM_COMMAND;
 	  break;
 	}
-      tuiSetLayout (new_layout, regs_type);
+      tui_set_layout (new_layout, regs_type);
     }
   return 0;
 }
@@ -215,18 +213,18 @@ tui_rl_delete_other_windows (int notused1, int notused2)
 static int
 tui_rl_other_window (int count, int key)
 {
-  TuiWinInfoPtr winInfo;
+  struct tui_win_info *win_info;
 
   if (!tui_active)
-    tui_rl_switch_mode (0/*notused*/, 0/*notused*/);
+    tui_rl_switch_mode (0 /* notused */, 0 /* notused */);
 
-  winInfo = tuiNextWin (tuiWinWithFocus ());
-  if (winInfo)
+  win_info = tui_next_win (tui_win_with_focus ());
+  if (win_info)
     {
-      tuiSetWinFocusTo (winInfo);
-      if (dataWin && dataWin->generic.isVisible)
-        tuiRefreshDataWin ();
-      keypad (cmdWin->generic.handle, (winInfo != cmdWin));
+      tui_set_win_focus_to (win_info);
+      if (TUI_DATA_WIN && TUI_DATA_WIN->generic.is_visible)
+        tui_refresh_data_win ();
+      keypad (TUI_CMD_WIN->generic.handle, (win_info != TUI_CMD_WIN));
     }
   return 0;
 }
@@ -243,9 +241,9 @@ tui_rl_command_key (int count, int key)
     {
       if (tui_commands[i].key == key)
         {
-          /* Must save the command because it can be modified
-             by execute_command.  */
-          char* cmd = alloca (strlen (tui_commands[i].cmd) + 1);
+          /* Must save the command because it can be modified by
+             execute_command.  */
+          char *cmd = alloca (strlen (tui_commands[i].cmd) + 1);
           strcpy (cmd, tui_commands[i].cmd);
           execute_command (cmd, TRUE);
           return 0;
@@ -261,7 +259,7 @@ tui_rl_command_key (int count, int key)
 static int
 tui_rl_command_mode (int count, int key)
 {
-  tui_set_key_mode (tui_one_command_mode);
+  tui_set_key_mode (TUI_ONE_COMMAND_MODE);
   return rl_insert (count, key);
 }
 
@@ -271,10 +269,10 @@ static int
 tui_rl_next_keymap (int notused1, int notused2)
 {
   if (!tui_active)
-    tui_rl_switch_mode (0/*notused*/, 0/*notused*/);
+    tui_rl_switch_mode (0 /* notused */, 0 /* notused */);
 
-  tui_set_key_mode (tui_current_key_mode == tui_command_mode
-                    ? tui_single_key_mode : tui_command_mode);
+  tui_set_key_mode (tui_current_key_mode == TUI_COMMAND_MODE
+                    ? TUI_SINGLE_KEY_MODE : TUI_COMMAND_MODE);
   return 0;
 }
 
@@ -283,29 +281,30 @@ tui_rl_next_keymap (int notused1, int notused2)
    the command window is cleaner.  It will be displayed if
    we temporarily leave the SingleKey mode.  */
 static int
-tui_rl_startup_hook ()
+tui_rl_startup_hook (void)
 {
   rl_already_prompted = 1;
-  if (tui_current_key_mode != tui_command_mode)
-    tui_set_key_mode (tui_single_key_mode);
+  if (tui_current_key_mode != TUI_COMMAND_MODE)
+    tui_set_key_mode (TUI_SINGLE_KEY_MODE);
   tui_redisplay_readline ();
   return 0;
 }
 
-/* Change the TUI key mode by installing the appropriate readline keymap.  */
+/* Change the TUI key mode by installing the appropriate readline
+   keymap.  */
 void
 tui_set_key_mode (enum tui_key_mode mode)
 {
   tui_current_key_mode = mode;
-  rl_set_keymap (mode == tui_single_key_mode
+  rl_set_keymap (mode == TUI_SINGLE_KEY_MODE
                  ? tui_keymap : tui_readline_standard_keymap);
-  tuiShowLocatorContent ();
+  tui_show_locator_content ();
 }
 
 /* Initialize readline and configure the keymap for the switching
    key shortcut.  */
 void
-tui_initialize_readline ()
+tui_initialize_readline (void)
 {
   int i;
   Keymap tui_ctlx_keymap;
@@ -379,20 +378,20 @@ tui_enable (void)
   
       cbreak ();
       noecho ();
-      /*timeout (1);*/
+      /* timeout (1); */
       nodelay(w, FALSE);
       nl();
       keypad (w, TRUE);
       rl_initialize ();
-      setTermHeightTo (LINES);
-      setTermWidthTo (COLS);
+      tui_set_term_height_to (LINES);
+      tui_set_term_width_to (COLS);
       def_prog_mode ();
 
-      tuiShowFrameInfo (0);
-      tuiSetLayout (SRC_COMMAND, TUI_UNDEFINED_REGS);
-      tuiSetWinFocusTo (srcWin);
-      keypad (cmdWin->generic.handle, TRUE);
-      wrefresh (cmdWin->generic.handle);
+      tui_show_frame_info (0);
+      tui_set_layout (SRC_COMMAND, TUI_UNDEFINED_REGS);
+      tui_set_win_focus_to (TUI_SRC_WIN);
+      keypad (TUI_CMD_WIN->generic.handle, TRUE);
+      wrefresh (TUI_CMD_WIN->generic.handle);
       tui_finish_init = 0;
     }
   else
@@ -412,12 +411,12 @@ tui_enable (void)
   tui_setup_io (1);
 
   tui_active = 1;
-  if (deprecated_selected_frame)
-     tuiShowFrameInfo (deprecated_selected_frame);
+  if (deprecated_safe_get_selected_frame ())
+     tui_show_frame_info (deprecated_safe_get_selected_frame ());
 
   /* Restore TUI keymap.  */
   tui_set_key_mode (tui_current_key_mode);
-  tuiRefreshAll ();
+  tui_refresh_all_win ();
 
   /* Update gdb's knowledge of its terminal.  */
   target_terminal_save_ours ();
@@ -456,31 +455,21 @@ tui_disable (void)
   tui_update_gdb_sizes ();
 }
 
-/* Wrapper on top of free() to ensure that input address
-   is greater than 0x0.  */
 void
-tuiFree (char *ptr)
+strcat_to_buf (char *buf, int buflen, 
+	       const char *item_to_add)
 {
-  if (ptr != (char *) NULL)
+  if (item_to_add != (char *) NULL && buf != (char *) NULL)
     {
-      xfree (ptr);
-    }
-}
-
-void
-strcat_to_buf (char *buf, int buflen, const char *itemToAdd)
-{
-  if (itemToAdd != (char *) NULL && buf != (char *) NULL)
-    {
-      if ((strlen (buf) + strlen (itemToAdd)) <= buflen)
-	strcat (buf, itemToAdd);
+      if ((strlen (buf) + strlen (item_to_add)) <= buflen)
+	strcat (buf, item_to_add);
       else
-	strncat (buf, itemToAdd, (buflen - strlen (buf)));
+	strncat (buf, item_to_add, (buflen - strlen (buf)));
     }
 }
 
 #if 0
-/* Solaris <sys/termios.h> defines CTRL. */
+/* Solaris <sys/termios.h> defines CTRL.  */
 #ifndef CTRL
 #define CTRL(x)         (x & ~0140)
 #endif
@@ -489,17 +478,15 @@ strcat_to_buf (char *buf, int buflen, const char *itemToAdd)
 #define CHK(val, dft)   (val<=0 ? dft : val)
 
 static void
-_tuiReset (void)
+tui_reset (void)
 {
   struct termio mode;
 
-  /*
-     ** reset the teletype mode bits to a sensible state.
-     ** Copied tset.c
-   */
-#if ! defined (USG) && defined (TIOCGETC)
+  /* Reset the teletype mode bits to a sensible state.
+     Copied tset.c.  */
+#if defined (TIOCGETC)
   struct tchars tbuf;
-#endif /* !USG && TIOCGETC */
+#endif /* TIOCGETC */
 #ifdef UCB_NTTY
   struct ltchars ltc;
 
@@ -515,7 +502,6 @@ _tuiReset (void)
       ioctl (FILEDES, TIOCSLTC, &ltc);
     }
 #endif /* UCB_NTTY */
-#ifndef USG
 #ifdef TIOCGETC
   ioctl (FILEDES, TIOCGETC, &tbuf);
   tbuf.t_intrc = CHK (tbuf.t_intrc, CTRL ('?'));
@@ -523,7 +509,7 @@ _tuiReset (void)
   tbuf.t_startc = CHK (tbuf.t_startc, CTRL ('Q'));
   tbuf.t_stopc = CHK (tbuf.t_stopc, CTRL ('S'));
   tbuf.t_eofc = CHK (tbuf.t_eofc, CTRL ('D'));
-  /* brkc is left alone */
+  /* brkc is left alone.  */
   ioctl (FILEDES, TIOCSETC, &tbuf);
 #endif /* TIOCGETC */
   mode.sg_flags &= ~(RAW
@@ -532,71 +518,51 @@ _tuiReset (void)
 #endif /* CBREAK */
 		     | VTDELAY | ALLDELAY);
   mode.sg_flags |= XTABS | ECHO | CRMOD | ANYP;
-#else /*USG */
-  ioctl (FILEDES, TCGETA, &mode);
-  mode.c_cc[VINTR] = CHK (mode.c_cc[VINTR], CTRL ('?'));
-  mode.c_cc[VQUIT] = CHK (mode.c_cc[VQUIT], CTRL ('\\'));
-  mode.c_cc[VEOF] = CHK (mode.c_cc[VEOF], CTRL ('D'));
-
-  mode.c_iflag &= ~(IGNBRK | PARMRK | INPCK | INLCR | IGNCR | IUCLC | IXOFF);
-  mode.c_iflag |= (BRKINT | ISTRIP | ICRNL | IXON);
-  mode.c_oflag &= ~(OLCUC | OCRNL | ONOCR | ONLRET | OFILL | OFDEL |
-		    NLDLY | CRDLY | TABDLY | BSDLY | VTDLY | FFDLY);
-  mode.c_oflag |= (OPOST | ONLCR);
-  mode.c_cflag &= ~(CSIZE | PARODD | CLOCAL);
-#ifndef hp9000s800
-  mode.c_cflag |= (CS8 | CREAD);
-#else /*hp9000s800 */
-  mode.c_cflag |= (CS8 | CSTOPB | CREAD);
-#endif /* hp9000s800 */
-  mode.c_lflag &= ~(XCASE | ECHONL | NOFLSH);
-  mode.c_lflag |= (ISIG | ICANON | ECHO | ECHOK);
-  ioctl (FILEDES, TCSETAW, &mode);
-#endif /* USG */
 
   return;
-}				/* _tuiReset */
+}
 #endif
 
 void
 tui_show_source (const char *file, int line)
 {
   struct symtab_and_line cursal = get_current_source_symtab_and_line ();
-  /* make sure that the source window is displayed */
-  tuiAddWinToLayout (SRC_WIN);
+  /* Make sure that the source window is displayed.  */
+  tui_add_win_to_layout (SRC_WIN);
 
-  tuiUpdateSourceWindowsWithLine (cursal.symtab, line);
-  tuiUpdateLocatorFilename (file);
+  tui_update_source_windows_with_line (cursal.symtab, line);
+  tui_update_locator_filename (file);
 }
 
 void
 tui_show_assembly (CORE_ADDR addr)
 {
-  tuiAddWinToLayout (DISASSEM_WIN);
-  tuiUpdateSourceWindowsWithAddr (addr);
+  tui_add_win_to_layout (DISASSEM_WIN);
+  tui_update_source_windows_with_addr (addr);
 }
 
 int
-tui_is_window_visible (TuiWinType type)
+tui_is_window_visible (enum tui_win_type type)
 {
   if (tui_active == 0)
     return 0;
 
-  if (winList[type] == 0)
+  if (tui_win_list[type] == 0)
     return 0;
   
-  return winList[type]->generic.isVisible;
+  return tui_win_list[type]->generic.is_visible;
 }
 
 int
-tui_get_command_dimension (int *width, int *height)
+tui_get_command_dimension (unsigned int *width, 
+			   unsigned int *height)
 {
-  if (!tui_active || !m_winPtrNotNull (cmdWin))
+  if (!tui_active || (TUI_CMD_WIN == NULL))
     {
       return 0;
     }
   
-  *width = cmdWin->generic.width;
-  *height = cmdWin->generic.height;
+  *width = TUI_CMD_WIN->generic.width;
+  *height = TUI_CMD_WIN->generic.height;
   return 1;
 }
