@@ -1,27 +1,28 @@
 /* CRIS exception, interrupt, and trap (EIT) support
-   Copyright (C) 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
    Contributed by Axis Communications.
 
 This file is part of the GNU simulators.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License along
-with this program; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "sim-main.h"
 #include "sim-options.h"
-#include "targ-vals.h"
 #include "bfd.h"
+/* FIXME: get rid of targ-vals.h usage everywhere else.  */
+
+#include <stdarg.h>
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -37,6 +38,21 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+/* For PATH_MAX, originally. */
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
+
+/* From ld/sysdep.h.  */
+#ifdef PATH_MAX
+# define SIM_PATHMAX PATH_MAX
+#else
+# ifdef MAXPATHLEN
+#  define SIM_PATHMAX MAXPATHLEN
+# else
+#  define SIM_PATHMAX 1024
+# endif
+#endif
 
 /* The verbatim values are from asm-cris/unistd.h.  */
 
@@ -63,6 +79,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define TARGET_SYS_truncate 92
 #define TARGET_SYS_ftruncate 93
 #define TARGET_SYS_socketcall 102
+#define TARGET_SYS_stat 106
 #define TARGET_SYS_fstat 108
 #define TARGET_SYS_wait4 114
 #define TARGET_SYS_sigreturn 119
@@ -223,6 +240,9 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define TARGET___WALL 0x40000000
 #define TARGET___WCLONE 0x80000000
 
+/* From linux/limits.h. */
+#define TARGET_PIPE_BUF 4096
+
 static const char stat_map[] =
 "st_dev,2:space,10:space,4:st_mode,4:st_nlink,4:st_uid,4"
 ":st_gid,4:st_rdev,2:space,10:st_size,8:st_blksize,4:st_blocks,4"
@@ -242,7 +262,6 @@ static const CB_TARGET_DEFS_MAP syscall_map[] =
   { CB_SYS_lstat, TARGET_SYS_lstat64 },
   { CB_SYS_stat, TARGET_SYS_stat64 },
   { CB_SYS_pipe, TARGET_SYS_pipe },
-  { CB_SYS_time, TARGET_SYS_time },
   { CB_SYS_rename, TARGET_SYS_rename },
   { CB_SYS_truncate, TARGET_SYS_truncate },
   { CB_SYS_ftruncate, TARGET_SYS_ftruncate },
@@ -260,6 +279,7 @@ static const char stat32_map[] =
 static const CB_TARGET_DEFS_MAP syscall_stat32_map[] =
 {
   { CB_SYS_fstat, TARGET_SYS_fstat },
+  { CB_SYS_stat, TARGET_SYS_stat },
   { 0, -1 }
 };
 
@@ -661,15 +681,22 @@ static const CB_TARGET_DEFS_MAP errno_map[] =
    installation and removing synonyms and unnecessary items.  Don't
    forget the end-marker.  */
 
+/* These we treat specially, as they're used in the fcntl F_GETFL
+   syscall.  For consistency, open_map is also manually edited to use
+   these macros.  */
+#define TARGET_O_ACCMODE 0x3
+#define TARGET_O_RDONLY 0x0
+#define TARGET_O_WRONLY 0x1
+
 static const CB_TARGET_DEFS_MAP open_map[] = {
 #ifdef O_ACCMODE
-  { O_ACCMODE, 0x3 },
+  { O_ACCMODE, TARGET_O_ACCMODE },
 #endif
 #ifdef O_RDONLY
-  { O_RDONLY, 0x0 },
+  { O_RDONLY, TARGET_O_RDONLY },
 #endif
 #ifdef O_WRONLY
-  { O_WRONLY, 0x1 },
+  { O_WRONLY, TARGET_O_WRONLY },
 #endif
 #ifdef O_RDWR
   { O_RDWR, 0x2 },
@@ -842,25 +869,25 @@ dump_statistics (SIM_CPU *current_cpu)
 
   /* For v32, unaligned_mem_dword_count should always be 0.  For
      v10, memsrc_stall_count should always be 0.  */
-  sim_io_eprintf (sd, "Memory source stall cycles: %lld\n",
-		  profp->memsrc_stall_count
-		  + profp->unaligned_mem_dword_count);
-  sim_io_eprintf (sd, "Memory read-after-write stall cycles: %lld\n",
-		  profp->memraw_stall_count);
-  sim_io_eprintf (sd, "Movem source stall cycles: %lld\n",
-		  profp->movemsrc_stall_count);
-  sim_io_eprintf (sd, "Movem destination stall cycles: %lld\n",
-		  profp->movemdst_stall_count);
-  sim_io_eprintf (sd, "Movem address stall cycles: %lld\n",
-		  profp->movemaddr_stall_count);
-  sim_io_eprintf (sd, "Multiplication source stall cycles: %lld\n",
-		  profp->mulsrc_stall_count);
-  sim_io_eprintf (sd, "Jump source stall cycles: %lld\n",
-		  profp->jumpsrc_stall_count);
-  sim_io_eprintf (sd, "Branch misprediction stall cycles: %lld\n",
-		  profp->branch_stall_count);
-  sim_io_eprintf (sd, "Jump target stall cycles: %lld\n",
-		  profp->jumptarget_stall_count);
+  sim_io_eprintf (sd, "Memory source stall cycles: %llu\n",
+		  (unsigned long long) (profp->memsrc_stall_count
+					+ profp->unaligned_mem_dword_count));
+  sim_io_eprintf (sd, "Memory read-after-write stall cycles: %llu\n",
+		  (unsigned long long) profp->memraw_stall_count);
+  sim_io_eprintf (sd, "Movem source stall cycles: %llu\n",
+		  (unsigned long long) profp->movemsrc_stall_count);
+  sim_io_eprintf (sd, "Movem destination stall cycles: %llu\n",
+		  (unsigned long long) profp->movemdst_stall_count);
+  sim_io_eprintf (sd, "Movem address stall cycles: %llu\n",
+		  (unsigned long long) profp->movemaddr_stall_count);
+  sim_io_eprintf (sd, "Multiplication source stall cycles: %llu\n",
+		  (unsigned long long) profp->mulsrc_stall_count);
+  sim_io_eprintf (sd, "Jump source stall cycles: %llu\n",
+		  (unsigned long long) profp->jumpsrc_stall_count);
+  sim_io_eprintf (sd, "Branch misprediction stall cycles: %llu\n",
+		  (unsigned long long) profp->branch_stall_count);
+  sim_io_eprintf (sd, "Jump target stall cycles: %llu\n",
+		  (unsigned long long) profp->jumptarget_stall_count);
 }
 
 /* Check whether any part of [addr .. addr + len - 1] is already mapped.
@@ -1322,6 +1349,31 @@ make_first_thread (SIM_CPU *current_cpu)
     abort ();
 }
 
+/* Handle unknown system calls.  Returns (if it does) the syscall
+   return value.  */
+
+static USI
+cris_unknown_syscall (SIM_CPU *current_cpu, USI pc, char *s, ...)
+{
+  SIM_DESC sd = CPU_STATE (current_cpu);
+  host_callback *cb = STATE_CALLBACK (sd);
+
+  if (cris_unknown_syscall_action == CRIS_USYSC_MSG_STOP
+      || cris_unknown_syscall_action == CRIS_USYSC_MSG_ENOSYS)
+    {
+      va_list ap;
+
+      va_start (ap, s);
+      sim_io_evprintf (sd, s, ap);
+      va_end (ap);
+
+      if (cris_unknown_syscall_action == CRIS_USYSC_MSG_STOP)
+	sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped, SIM_SIGILL);
+    }
+
+  return -cb_host_to_target_errno (cb, ENOSYS);
+}
+
 /* Main function: the handler of the "break 13" syscall insn.  */
 
 USI
@@ -1397,8 +1449,9 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 
 	case TARGET_SYS_fcntl64:
 	case TARGET_SYS_fcntl:
-	  if (arg2 == 1)
+	  switch (arg2)
 	    {
+	    case 1:
 	      /* F_GETFD.
 		 Glibc checks stdin, stdout and stderr fd:s for
 		 close-on-exec security sanity.  We just need to provide a
@@ -1406,12 +1459,51 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		 close-on-exec flag true, we could just do a real fcntl
 		 here.  */
 	      retval = 0;
-	    }
-	  else if (arg2 == 2)
-	    {
+	      break;
+
+	    case 2:
 	      /* F_SETFD.  Just ignore attempts to set the close-on-exec
 		 flag.  */
 	      retval = 0;
+	      break;
+
+	    case 3:
+	      /* F_GETFL.  Check for the special case for open+fdopen.  */
+	      if (current_cpu->last_syscall == TARGET_SYS_open
+		  && arg1 == current_cpu->last_open_fd)
+		{
+		  retval = current_cpu->last_open_flags & TARGET_O_ACCMODE;
+		  break;
+		}
+	      else if (arg1 == 0)
+		{
+		  /* Because we can't freopen fd:s 0, 1, 2 to mean
+		     something else than stdin, stdout and stderr
+		     (sim/common/syscall.c:cb_syscall special cases fd
+		     0, 1 and 2), we know what flags that we can
+		     sanely return for these fd:s.  */
+		  retval = TARGET_O_RDONLY;
+		  break;
+		}
+	      else if (arg1 == 1 || arg1 == 2)
+		{
+		  retval = TARGET_O_WRONLY;
+		  break;
+		}
+	      /* FALLTHROUGH */
+	    default:
+	      /* Nothing else is implemented.  */
+	      retval
+		= cris_unknown_syscall (current_cpu, pc,
+					"Unimplemented %s syscall "
+					"(fd: 0x%lx: cmd: 0x%lx arg: "
+					"0x%lx)\n",
+					callnum == TARGET_SYS_fcntl
+					? "fcntl" : "fcntl64",
+					(unsigned long) (USI) arg1,
+					(unsigned long) (USI) arg2,
+					(unsigned long) (USI) arg3);
+	      break;
 	    }
 	  break;
 
@@ -1532,16 +1624,17 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		|| (fd != (USI) -1 && prot != TARGET_PROT_READ)
 		|| pgoff != 0)
 	      {
-		sim_io_eprintf (sd, "Unimplemented mmap2 call "
-				"(0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx)\n",
-				(unsigned long) arg1,
-				(unsigned long) arg2,
-				(unsigned long) arg3,
-				(unsigned long) arg4,
-				(unsigned long) arg5,
-				(unsigned long) arg6);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+						 "Unimplemented mmap2 call "
+						 "(0x%lx, 0x%lx, 0x%lx, "
+						 "0x%lx, 0x%lx, 0x%lx)\n",
+						 (unsigned long) arg1,
+						 (unsigned long) arg2,
+						 (unsigned long) arg3,
+						 (unsigned long) arg4,
+						 (unsigned long) arg5,
+						 (unsigned long) arg6);
 		break;
 	      }
 	    else if (fd != (USI) -1)
@@ -1635,13 +1728,13 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		|| !is_mapped (sd, &current_cpu->highest_mmapped_page, addr,
 			       len))
 	      {
-		sim_io_eprintf (sd, "Unimplemented mprotect call "
-				"(0x%lx, 0x%lx, 0x%lx)\n",
-				(unsigned long) arg1,
-				(unsigned long) arg2,
-				(unsigned long) arg3);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Unimplemented mprotect call "
+					  "(0x%lx, 0x%lx, 0x%lx)\n",
+					  (unsigned long) arg1,
+					  (unsigned long) arg2,
+					  (unsigned long) arg3);
 		break;
 	      }
 
@@ -1700,14 +1793,14 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		|| rusagep != 0
 		|| current_cpu->thread_data == NULL)
 	      {
-		sim_io_eprintf (sd, "Unimplemented wait4 call "
-				"(0x%lx, 0x%lx, 0x%lx, 0x%lx)\n",
-				(unsigned long) arg1,
-				(unsigned long) arg2,
-				(unsigned long) arg3,
-				(unsigned long) arg4);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Unimplemented wait4 call "
+					  "(0x%lx, 0x%lx, 0x%lx, 0x%lx)\n",
+					  (unsigned long) arg1,
+					  (unsigned long) arg2,
+					  (unsigned long) arg3,
+					  (unsigned long) arg4);
 		break;
 	      }
 
@@ -1813,19 +1906,22 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		     && target_sa_flags != (TARGET_SA_RESTART|TARGET_SA_SIGINFO))
 		    || target_sa_handler == 0)
 		  {
-		    sim_io_eprintf (sd, "Unimplemented rt_sigaction "
-				    "syscall (0x%lx, "
-				    "0x%lx: [0x%x, 0x%x, 0x%x, "
-				    "{0x%x, 0x%x}], "
-				    "0x%lx)\n",
-				    (unsigned long) arg1,
-				    (unsigned long) arg2,
-				    target_sa_handler, target_sa_flags,
-				    target_sa_restorer,
-				    target_sa_mask_low, target_sa_mask_high,
-				    (unsigned long) arg3);
-		    sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				     SIM_SIGILL);
+		    retval
+		      = cris_unknown_syscall (current_cpu, pc,
+					      "Unimplemented rt_sigaction "
+					      "syscall "
+					      "(0x%lx, 0x%lx: "
+					      "[0x%x, 0x%x, 0x%x, "
+					      "{0x%x, 0x%x}], 0x%lx)\n",
+					      (unsigned long) arg1,
+					      (unsigned long) arg2,
+					      target_sa_handler,
+					      target_sa_flags,
+					      target_sa_restorer,
+					      target_sa_mask_low,
+					      target_sa_mask_high,
+					      (unsigned long) arg3);
+		    break;
 		  }
 
 		current_cpu->sighandler[signum] = target_sa_handler;
@@ -1946,11 +2042,14 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		    || (buf.st_mode & S_IFIFO) == 0)
 		|| current_cpu->thread_data == NULL)
 	      {
-		sim_io_eprintf (sd, "Unimplemented poll syscall "
-				"(0x%lx: [0x%x, 0x%x, x], 0x%lx, 0x%lx)\n",
-				(unsigned long) arg1, fd, events,
-				(unsigned long) arg2, (unsigned long) arg3);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped, SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Unimplemented poll syscall "
+					  "(0x%lx: [0x%x, 0x%x, x], "
+					  "0x%lx, 0x%lx)\n",
+					  (unsigned long) arg1, fd, events,
+					  (unsigned long) arg2,
+					  (unsigned long) arg3);
 		break;
 	      }
 
@@ -1995,6 +2094,17 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 	    break;
 	  }
 
+	case TARGET_SYS_time:
+	  {
+	    retval = (int) (*cb->time) (cb, 0L);
+
+	    /* At time of this writing, CB_SYSCALL_time doesn't do the
+	       part of setting *arg1 to the return value.  */
+	    if (arg1)
+	      sim_core_write_unaligned_4 (current_cpu, pc, 0, arg1, retval);
+	    break;
+	  }
+
 	case TARGET_SYS_gettimeofday:
 	  if (arg1 != 0)
 	    {
@@ -2031,12 +2141,13 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 	    if (!((offs_hi == 0 && offs_lo >= 0)
 		  || (offs_hi == -1 &&  offs_lo < 0)))
 	      {
-		sim_io_eprintf (sd,
-				"Unimplemented llseek offset,"
-				" fd %d: 0x%x:0x%x\n",
-				fd, (unsigned) arg2, (unsigned) arg3);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Unimplemented llseek offset,"
+					  " fd %d: 0x%x:0x%x\n",
+					  fd, (unsigned) arg2,
+					  (unsigned) arg3);
+		break;
 	      }
 
 	    s.func = TARGET_SYS_lseek;
@@ -2114,11 +2225,11 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		&& how != TARGET_SIG_SETMASK
 		&& how != TARGET_SIG_UNBLOCK)
 	      {
-		sim_io_eprintf (sd, "Unimplemented rt_sigprocmask syscall "
-			       "(0x%x, 0x%x, 0x%x)\n", arg1, arg2, arg3);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
-		retval = 0;
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Unimplemented rt_sigprocmask "
+					  "syscall (0x%x, 0x%x, 0x%x)\n",
+					  arg1, arg2, arg3);
 		break;
 	      }
 
@@ -2189,17 +2300,19 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		|| (current_cpu->thread_data[threadno].cpu_context_atsignal
 		    == NULL))
 	      {
-		sim_io_eprintf (sd, "Invalid sigreturn syscall: no signal"
-				" handler active "
-				"(0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx)\n",
-				(unsigned long) arg1,
-				(unsigned long) arg2,
-				(unsigned long) arg3,
-				(unsigned long) arg4,
-				(unsigned long) arg5,
-				(unsigned long) arg6);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Invalid sigreturn syscall: "
+					  "no signal handler active "
+					  "(0x%lx, 0x%lx, 0x%lx, 0x%lx, "
+					  "0x%lx, 0x%lx)\n",
+					  (unsigned long) arg1,
+					  (unsigned long) arg2,
+					  (unsigned long) arg3,
+					  (unsigned long) arg4,
+					  (unsigned long) arg5,
+					  (unsigned long) arg6);
+		break;
 	      }
 
 	    was_sigsuspended
@@ -2265,11 +2378,13 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 
 	    if (setsize != 8)
 	      {
-		sim_io_eprintf (sd, "Unimplemented rt_sigsuspend syscall"
-			       " arguments (0x%lx, 0x%lx)\n",
-				(unsigned long) arg1, (unsigned long) arg2);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Unimplemented rt_sigsuspend syscall"
+					  " arguments (0x%lx, 0x%lx)\n",
+					  (unsigned long) arg1,
+					  (unsigned long) arg2);
+		break;
 	      }
 
 	    /* Don't change the signal mask if we're already in
@@ -2324,6 +2439,7 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 	  /* Add case labels here for other syscalls using the 32-bit
 	     "struct stat", provided they have a corresponding simulator
 	     function of course.  */
+	case TARGET_SYS_stat:
 	case TARGET_SYS_fstat:
 	  {
 	    /* As long as the infrastructure doesn't cache anything
@@ -2353,8 +2469,8 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 	    USI buf = arg1;
 	    USI size = arg2;
 
-	    char *cwd = xmalloc (MAXPATHLEN);
-	    if (cwd != getcwd (cwd, MAXPATHLEN))
+	    char *cwd = xmalloc (SIM_PATHMAX);
+	    if (cwd != getcwd (cwd, SIM_PATHMAX))
 	      abort ();
 
 	    /* FIXME: When and if we support chdir, we need something
@@ -2380,8 +2496,8 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 	    SI path = arg1;
 	    SI buf = arg2;
 	    SI bufsiz = arg3;
-	    char *pbuf = xmalloc (MAXPATHLEN);
-	    char *lbuf = xmalloc (MAXPATHLEN);
+	    char *pbuf = xmalloc (SIM_PATHMAX);
+	    char *lbuf = xmalloc (SIM_PATHMAX);
 	    char *lbuf_alloc = lbuf;
 	    int nchars = -1;
 	    int i;
@@ -2393,7 +2509,7 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		o += strlen (simulator_sysroot);
 	      }
 
-	    for (i = 0; i + o < MAXPATHLEN; i++)
+	    for (i = 0; i + o < SIM_PATHMAX; i++)
 	      {
 		pbuf[i + o]
 		  = sim_core_read_unaligned_1 (current_cpu, pc, 0, path + i);
@@ -2401,7 +2517,7 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		  break;
 	      }
 
-	    if (i + o == MAXPATHLEN)
+	    if (i + o == SIM_PATHMAX)
 	      {
 		retval = -cb_host_to_target_errno (cb, ENAMETOOLONG);
 		break;
@@ -2418,12 +2534,12 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 
 		if (argv0 == NULL || *argv0 == '.')
 		  {
-		    sim_io_eprintf (sd, "Unimplemented readlink syscall "
-				    "(0x%lx: [\"%s\"], 0x%lx)\n",
-				    (unsigned long) arg1, pbuf,
-				    (unsigned long) arg2);
-		    sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				     SIM_SIGILL);
+		    retval
+		      = cris_unknown_syscall (current_cpu, pc,
+					      "Unimplemented readlink syscall "
+					      "(0x%lx: [\"%s\"], 0x%lx)\n",
+					      (unsigned long) arg1, pbuf,
+					      (unsigned long) arg2);
 		    break;
 		  }
 		else if (*argv0 == '/')
@@ -2437,8 +2553,8 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		  }
 		else
 		  {
-		    if (getcwd (lbuf, MAXPATHLEN) != NULL
-			&& strlen (lbuf) + 2 + strlen (argv0) < MAXPATHLEN)
+		    if (getcwd (lbuf, SIM_PATHMAX) != NULL
+			&& strlen (lbuf) + 2 + strlen (argv0) < SIM_PATHMAX)
 		      {
 			if (strncmp (simulator_sysroot, lbuf,
 				     strlen (simulator_sysroot)) == 0)
@@ -2453,7 +2569,7 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		  }
 	      }
 	    else
-	      nchars = readlink (pbuf, lbuf, MAXPATHLEN);
+	      nchars = readlink (pbuf, lbuf, SIM_PATHMAX);
 
 	    /* We trust that the readlink result returns a *relative*
 	       link, or one already adjusted for the file-path-prefix.
@@ -2656,19 +2772,19 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		break;
 	      }
 
-	    sim_io_eprintf (sd, "Unimplemented _sysctl syscall "
-			    "(0x%lx: [0x%lx, 0x%lx],"
-			    " 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx)\n",
-			    (unsigned long) name,
-			    (unsigned long) name0,
-			    (unsigned long) name1,
-			    (unsigned long) nlen,
-			    (unsigned long) oldval,
-			    (unsigned long) oldlenp,
-			    (unsigned long) newval,
-			    (unsigned long) newlen);
-	    sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-			     SIM_SIGILL);
+	    retval
+	      = cris_unknown_syscall (current_cpu, pc,
+				      "Unimplemented _sysctl syscall "
+				      "(0x%lx: [0x%lx, 0x%lx],"
+				      " 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx)\n",
+				      (unsigned long) name,
+				      (unsigned long) name0,
+				      (unsigned long) name1,
+				      (unsigned long) nlen,
+				      (unsigned long) oldval,
+				      (unsigned long) oldlenp,
+				      (unsigned long) newval,
+				      (unsigned long) newlen);
 	    break;
 	  }
 
@@ -2741,11 +2857,13 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 		 | TARGET_CLONE_SIGHAND)
 		|| newsp == 0)
 	      {
-		sim_io_eprintf (sd,
-				"Unimplemented clone syscall (0x%lx, 0x%lx)\n",
-				(unsigned long) arg1, (unsigned long) arg2);
-		sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-				 SIM_SIGILL);
+		retval
+		  = cris_unknown_syscall (current_cpu, pc,
+					  "Unimplemented clone syscall "
+					  "(0x%lx, 0x%lx)\n",
+					  (unsigned long) arg1,
+					  (unsigned long) arg2);
+		break;
 	      }
 
 	    if (current_cpu->thread_data == NULL)
@@ -2808,14 +2926,24 @@ cris_break_13_handler (SIM_CPU *current_cpu, USI callnum, USI arg1,
 
 	unimplemented_syscall:
 	default:
-	  sim_io_eprintf (sd, "Unimplemented syscall: %d "
-			  "(0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x)\n", callnum,
-			  arg1, arg2, arg3, arg4, arg5, arg6);
-	  sim_engine_halt (sd, current_cpu, NULL, pc, sim_stopped,
-			   SIM_SIGILL);
+	  retval
+	    = cris_unknown_syscall (current_cpu, pc,
+				    "Unimplemented syscall: %d "
+				    "(0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x)\n",
+				    callnum, arg1, arg2, arg3, arg4, arg5,
+				    arg6);
 	}
     }
 
+  /* Minimal support for fcntl F_GETFL as used in open+fdopen.  */
+  if (callnum == TARGET_SYS_open)
+    {
+      current_cpu->last_open_fd = retval;
+      current_cpu->last_open_flags = arg2;
+    }
+
+  current_cpu->last_syscall = callnum;
+
   /* A system call is a rescheduling point.  For the time being, we don't
      reschedule anywhere else.  */
   if (current_cpu->m1threads != 0
@@ -2888,13 +3016,14 @@ cris_pipe_nonempty (host_callback *cb ATTRIBUTE_UNUSED,
 
 static void
 cris_pipe_empty (host_callback *cb,
-		 int reader ATTRIBUTE_UNUSED,
+		 int reader,
 		 int writer)
 {
   int i;
   SIM_CPU *cpu = current_cpu_for_cb_callback;
   bfd_byte r10_buf[4];
-  int remaining = cb->pipe_buffer[writer].size;
+  int remaining
+    = cb->pipe_buffer[writer].size - cb->pipe_buffer[reader].size;
 
   /* We need to find the thread that waits for this pipe.  */
   for (i = 0; i < SIM_TARGET_MAX_THREADS; i++)
@@ -2902,6 +3031,7 @@ cris_pipe_empty (host_callback *cb,
 	&& cpu->thread_data[i].pipe_write_fd == writer)
       {
 	int retval;
+
 	/* Temporarily switch to this cpu context, so we can change the
 	   PC by ordinary calls.  */
 
@@ -2912,19 +3042,25 @@ cris_pipe_empty (host_callback *cb,
 		cpu->thread_data[i].cpu_context,
 		cpu->thread_cpu_data_size);
 
-	/* The return value is supposed to contain the number of written
-	   bytes, which is the number of bytes requested and returned at
-	   the write call.  We subtract the remaining bytes from that,
-	   but making sure we still get a positive number.
-	   The return value may also be a negative number; an error
-	   value.  We cover this case by comparing against remaining,
-	   which is always >= 0.  */
+	/* The return value is supposed to contain the number of
+	   written bytes, which is the number of bytes requested and
+	   returned at the write call.  You might think the right
+	   thing is to adjust the return-value to be only the
+	   *consumed* number of bytes, but it isn't.  We're only
+	   called if the pipe buffer is fully consumed or it is being
+	   closed, possibly with remaining bytes.  For the latter
+	   case, the writer is still supposed to see success for
+	   PIPE_BUF bytes (a constant which we happen to know and is
+	   unlikely to change).  The return value may also be a
+	   negative number; an error value.  This case is covered
+	   because "remaining" is always >= 0.  */
 	(*CPU_REG_FETCH (cpu)) (cpu, H_GR_R10, r10_buf, 4);
 	retval = (int) bfd_getl_signed_32 (r10_buf);
-	if (retval >= remaining)
-	  bfd_putl32 (retval - remaining, r10_buf);
-	(*CPU_REG_STORE (cpu)) (cpu, H_GR_R10, r10_buf, 4);
-
+	if (retval - remaining > TARGET_PIPE_BUF)
+	  {
+	    bfd_putl32 (retval - remaining, r10_buf);
+	    (*CPU_REG_STORE (cpu)) (cpu, H_GR_R10, r10_buf, 4);
+	  }
 	sim_pc_set (cpu, sim_pc_get (cpu) + 2);
 	memcpy (cpu->thread_data[i].cpu_context,
 		&cpu->cpu_data_placeholder,
